@@ -90,6 +90,8 @@ pub trait ResourceExt {
     fn owner(self, owner: metav1::OwnerReference) -> Self;
     #[must_use]
     fn labels(self, labels: impl IntoIterator<Item = (impl ToString, impl ToString)>) -> Self;
+    #[must_use]
+    fn with_resource_version(self, resource_version: String) -> Self;
 }
 
 impl<T> ResourceExt for T
@@ -114,4 +116,9 @@ where
         self.labels_mut().extend(labels);
         self
     }
+
+    fn with_resource_version(mut self, resource_version: String) -> Self {
+        self.meta_mut().resource_version = Some(resource_version);
+        self
+    }
 }
