@@ -12,9 +12,9 @@ pub trait ConfigMapExt: super::ResourceExt {
     #[must_use]
     fn immutable(self, yes: bool) -> Self;
     #[must_use]
-    fn binary_data(self, data: impl IntoIterator<Item = (String, ByteString)>) -> Self;
+    fn binary_data(self, data: impl IntoIterator<Item = (impl ToString, ByteString)>) -> Self;
     #[must_use]
-    fn data(self, data: impl IntoIterator<Item = (String, String)>) -> Self;
+    fn data(self, data: impl IntoIterator<Item = (impl ToString, impl ToString)>) -> Self;
 }
 
 impl ConfigMapExt for corev1::ConfigMap {
@@ -34,16 +34,22 @@ impl ConfigMapExt for corev1::ConfigMap {
         Self { immutable, ..self }
     }
 
-    fn binary_data(self, data: impl IntoIterator<Item = (String, ByteString)>) -> Self {
-        let data = data.into_iter().collect();
+    fn binary_data(self, data: impl IntoIterator<Item = (impl ToString, ByteString)>) -> Self {
+        let data = data
+            .into_iter()
+            .map(|(key, value)| (key.to_string(), value))
+            .collect();
         Self {
             binary_data: Some(data),
             ..self
         }
     }
 
-    fn data(self, data: impl IntoIterator<Item = (String, String)>) -> Self {
-        let data = data.into_iter().collect();
+    fn data(self, data: impl IntoIterator<Item = (impl ToString, impl ToString)>) -> Self {
+        let data = data
+            .into_iter()
+            .map(|(key, value)| (key.to_string(), value.to_string()))
+            .collect();
         Self {
             data: Some(data),
             ..self
