@@ -3,9 +3,12 @@
 // Use is subject to license terms.
 //
 
+use std::time::Duration;
+
 use serde_json as json;
 
 use super::*;
+use crate::ext::ResourceExt as _;
 
 impl super::Kubectl {
     pub fn delete_params(&self) -> api::DeleteParams {
@@ -203,27 +206,79 @@ impl super::Kubectl {
         name: &str,
         data: BTreeMap<String, String>,
     ) -> kube::Result<corev1::ConfigMap> {
-        let mut cm: corev1::ConfigMap = json::from_value(json::json!(
-            {
-                "metadata": {
-                    "name": name
-                },
-                "data": data
+        const NOT_FOUND: http::StatusCode = http::StatusCode::NOT_FOUND;
+
+        let pp = self.post_params();
+        let configmaps = self.configmaps(None);
+        let mut cm = corev1::ConfigMap::new(name).data(data);
+
+        match configmaps.get(name).await {
+            Ok(existing) => {
+                if let Some(version) = existing.resource_version() {
+                    cm = cm.with_resource_version(version);
+                }
+                configmaps.replace(name, &pp, &cm).await
             }
-        ))
-        .expect("Failed to create ConfigMap");
+            Err(kube::Error::Api(e)) if e.code == NOT_FOUND.as_u16() => {
+                configmaps.create(&pp, &cm).await
+            }
+            Err(e) => Err(e),
+        }
+    }
+
+    pub async fn update_config_map(
+        &self,
+        name: &str,
+        mut update_fn: impl FnMut(BTreeMap<String, String>) -> BTreeMap<String, String>,
+    ) -> kube::Result<corev1::ConfigMap> {
+        const NOT_FOUND: http::StatusCode = http::StatusCode::NOT_FOUND;
+        const CONFLICT: http::StatusCode = http::StatusCode::CONFLICT;
+        const INTERNAL_SERVER_ERROR: http::StatusCode = http::StatusCode::INTERNAL_SERVER_ERROR;
+        const RETRY_COUNT: u8 = 16;
 
         let pp = self.post_params();
         let configmaps = self.configmaps(None);
 
-        let cm = if let Ok(existing) = configmaps.get(name).await {
-            cm.meta_mut().resource_version = existing.resource_version();
-            configmaps.replace(name, &pp, &cm).await?
-        } else {
-            configmaps.create(&pp, &cm).await?
-        };
+        let mut last_err = None;
+        for retry in 0..RETRY_COUNT {
+            match configmaps.get(name).await {
+                Ok(mut existing) => {
+                    let data = std::mem::take(&mut existing.data).unwrap_or_default();
+                    let data = update_fn(data);
+                    let cm = if let Some(resource_version) = existing.resource_version() {
+                        corev1::ConfigMap::new(name)
+                            .data(data)
+                            .with_resource_version(resource_version)
+                    } else {
+                        continue;
+                    };
 
-        Ok(cm)
+                    match configmaps.replace(name, &pp, &cm).await {
+                        // if we got `CONFLICT` then either uid or resourceVersion do not match
+                        // sleep with a backoff and retry
+                        Err(kube::Error::Api(e)) if e.code == CONFLICT.as_u16() => {
+                            last_err = Some(e);
+                        }
+                        result => return result,
+                    }
+                }
+                Err(kube::Error::Api(e)) if e.code == NOT_FOUND.as_u16() => {
+                    let data = update_fn(BTreeMap::new());
+                    let cm = corev1::ConfigMap::new(name).data(data);
+                    return configmaps.create(&pp, &cm).await;
+                }
+                Err(e) => return Err(e),
+            }
+            tokio::time::sleep(Duration::from_millis((1 << retry) * 10)).await
+        }
+
+        let e = last_err.unwrap_or_else(|| kube::error::ErrorResponse {
+            code: INTERNAL_SERVER_ERROR.as_u16(),
+            status: INTERNAL_SERVER_ERROR.as_str().to_string(),
+            message: String::new(),
+            reason: String::new(),
+        });
+        Err(kube::Error::Api(e))
     }
 
     pub async fn list_daemonsets<'a>(
